<?php
/**
 * Archive Plugin: displays links to all wiki pages with a certain category tag or from a given month
 * 
 * @license  GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author   Esther Brunner <esther@kaffeehaus.ch>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_PLUGIN.'blog/functions.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_archive extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'esther@kaffeehaus.ch',
      'date'   => '2006-04-14',
      'name'   => 'Blog Plugin (archive component)',
      'desc'   => 'Displays a list of wiki pages from a given month or with a given category tag',
      'url'    => 'http://wiki.splitbrain.org/plugin:blog',
    );
  }

  function getType(){ return 'substition'; }
  function getPType(){ return 'block'; }
  function getSort(){ return 309; }
  function connectTo($mode) { $this->Lexer->addSpecialPattern('\{\{archive>.+?\}\}',$mode,'plugin_blog_archive'); }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match,10,-2); // strip {{archive> from start and }} from end
    list($ns, $rest) = explode("?", $match);
    if (!$rest){
      $rest = $ns;
      $ns   = '';
    }
    
    // monthly archive
    if (preg_match("/\d{4}-\d{2}/", $rest)){
      list($year, $month) = explode("-", $rest);
      return array('month', $ns, $month, $year);
    
    // category tag archive
    } else {
      return array('tag', $ns, $rest);
    }
    
    return false;
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data) {
    global $ID;
    global $conf;
    global $auth;
    
    $ns = $data[1];
    if ($ns == '') $ns = cleanID(blog_getConf('blog_namespace'));
    
    if ($data[0] == 'month'){
      $pages = $this->_monthArchive($ns, $data[2], $data[3]);
    } else {
      $pages = $this->_tagArchive($ns, $data[2]);
    }
    
    if (!count($pages)) return true; // nothing to display
    
    if ($mode == 'xhtml'){
      
      // prevent caching to ensure content is always fresh
      $renderer->info['cache'] = false;
  
      $renderer->doc .= '<table class="archive">';
      foreach ($pages as $page){
        $renderer->doc .= '<tr><td class="page">';
        $id  = $page['id'];
        $title = p_get_first_heading($id);
        $renderer->doc .= $renderer->internallink(':'.$id, $title).'</td>';
        if (blog_getConf('archive_showdate')){
          $renderer->doc .= '<td class="date">'.date($conf['dformat'],$page['date']).'</td>';
        }
        if ($page['user'] && blog_getConf('archive_showuser') && !is_null($auth)){
          $userInfo = $auth->getUserData($page['user']);
          $renderer->doc .= '<td class="user">'.$userInfo['name'].'</td>';
        }
        $renderer->doc .= '</tr>';
      }
      $renderer->doc .= '</table>';
      
      return true;
      
    // for metadata renderer
    } elseif ($mode == 'metadata'){
      foreach ($pages as $page){
        $id  = $page['id'];
        $renderer->meta['relation']['references'][$id] = true;
        // $meta = array('relation' => array('isreferencedby' => array($ID => true)));
        // p_set_metadata($id, $meta);
      }
      
      return true;
    }
    return false;
  }
  
  /**
   * return the monthly archive list
   */
  function _monthArchive($ns, $month, $year){
    require_once(DOKU_PLUGIN.'blog/inc/recent.php');

    $result = array();
    
    $nextmonth   = $month + 1;
    $year2     = $year;
    if ($nextmonth > 12){
      $nextmonth = 1;
      $year2   = $year + 1;
    }
    
    $monthstart  = mktime(0, 0, 0, $month, 1, $year);
    $monthend    = mktime(0, 0, 0, $nextmonth, 1, $year2);
    
    $flags  = RECENTS_SKIP_DELETED + RECENTS_SKIP_MINORS;
    if ($ns == '*') $ns = '';
    else $flags += RECENTS_SKIP_SUBSPACES;
    
    $result = blog_recents(0, 0, $ns, $flags, 'month', array($monthstart, $monthend));
    
    if (!count($result)) return $result;
        
    return $result;
  }
  
  /**
   * returns the category archive list; very similar to ft_backlinks()
   *
   * @author  Andreas Gohr <andi@splitbrain.org>
   * @author  Esther Brunner <esther@kaffeehaus.ch>
   */
  function _tagArchive($ns, $tag){
    require_once (DOKU_INC.'inc/indexer.php');
    require_once (DOKU_INC.'inc/fulltext.php');
    
    if ($ns == '*') $ns = '';
    $result = array();
    
    // quick lookup of the pagename
    $sw    = array(); // we don't use stopwords here
    $matches = idx_lookup(idx_tokenizer($tag, $sw));  // pagename may contain specials (_ or .)
    $docs  = array_keys(ft_resultCombine(array_values($matches)));
    $docs  = array_filter($docs,'isVisiblePage'); // discard hidden pages
    if (!count($docs)) return $result;

    // check instructions for matching links
    foreach ($docs as $match){
      $matchFN = wikiFN($match);
      if (strpos(':'.getNS($match), ':'.$ns) !== 0) continue;
      $instructions = p_cached_instructions($matchFN, true);
      if (is_null($instructions)) continue;
            
      foreach ($instructions as $ins){
        if (($ins[0] == 'plugin') && ($ins[1][0] == 'blog_tag')){
          foreach ($ins[1][1] as $word){
            if (utf8_strtolower($tag) == utf8_strtolower($word)){
              //we have a match - finish
              $result[] = array(
                'id'   => $match,
                'date' => filemtime($matchFN),
                'user' => ''
              );
              break 2;
            }
          }
        }
      }
    }
      
    if (!count($result)) return $result;
      
    // check ACL permissions
    foreach (array_keys($result) as $idx){
      if (auth_quickaclcheck($result[$idx]['id']) < AUTH_READ) unset($result[$idx]);
    }
    
    sort($result);
    return $result;
  }
        
}

//Setup VIM: ex: et ts=4 enc=utf-8 :
